#include <iostream>

#include "GWt_elementLigneFaiseau.h"
#include "nomdElements.h"
#include "GWt_console.h"
#include "abstractElement.h"
#include "GWt_rfgun.h"
#include "GWt_drift.h"
#include "GWt_cell.h"
#include "GWt_bend.h"
#include "GWt_soleno.h"
#include "GWt_beam.h"
#include "GWt_fit.h"


#include <Wt/WCssDecorationStyle>
#include <Wt/WSignalMapper>
#include <Wt/WText>
#include <Wt/WImage>
#include <Wt/WHBoxLayout>
#include <Wt/WGridLayout>
#include <Wt/WLength>
#include <Wt/WVBoxLayout>

#define PSPA_DEBUG 1

GWt_elementLigneFaiseau::GWt_elementLigneFaiseau()
  : WContainerWidget(),
    firstTimeInsertMessage_(NULL),
    arrowImage_(NULL),
    lineImage_(NULL),
    abstractElement_(NULL)
{
  initialize_ligneFaisceau(NULL);
}


GWt_elementLigneFaiseau::GWt_elementLigneFaiseau(abstractElement* abstractElement)
  : WContainerWidget(),
    firstTimeInsertMessage_(NULL),
    arrowImage_(NULL),
    lineImage_(NULL),
    abstractElement_(NULL),
    elementName_(NULL)
{
    initialize_ligneFaisceau(abstractElement);
}


void GWt_elementLigneFaiseau::initialize_ligneFaisceau(abstractElement* abstractElement)
{
    
    cout<<"new GWt_elementLigneFaiseau : "<< this << endl;
    
    // set layout
    //     +----------+--------+---------+
    //     |   (--->) |  ELEM  | (-----) |
    //     +----------+--------+---------+
    //     |             NAME            |
    //     +----------+--------+---------+
    
    WGridLayout *gridLayout = new WGridLayout();
    
    // add first line :  initialize arrow and link images
    arrowImage_ = new GWt_dropZoneLigneFaisceau();
    dragElementZone_ = new WContainerWidget();
    lineImage_ = new GWt_dropZoneLigneFaisceau();
    
    // change object name in order to manage css :
    arrowImage_->setStyleClass("elementLigneFaiseau_line");
    lineImage_->setStyleClass("elementLigneFaiseau_line");
    
    
    WContainerWidget* arrow = new WContainerWidget();
    new WImage(arrow);
    arrow->resize(11,30);
    
    arrow->setStyleClass("elementLigneFaiseau_arrow");
    arrowImage_->addWidget(arrow);
    
      gridLayout->addWidget	(arrowImage_,0,0);
     gridLayout->addWidget	(dragElementZone_,0,1);
     gridLayout->addWidget	(lineImage_,0,2);
    
  // add the image if present
     if (abstractElement != NULL) {
     typedElement eType = abstractElement->getNomdElement().getElementType();
 
     switch (  eType ) {
     case RFgun :
     abstractElement_ = new GWt_rfgun(abstractElement, this);
     break;
     case drift :
     abstractElement_ = new GWt_drift(abstractElement, this);
     break;
     case cell  :
     abstractElement_ = new GWt_cell(abstractElement, this);
     break;
     case bend  :
     abstractElement_ = new GWt_bend(abstractElement, this);
     break;
     case soleno  :
     abstractElement_ = new GWt_soleno(abstractElement, this);
     break;
     case beam  :
     abstractElement_ = new GWt_beam(abstractElement, this);
     break;
     case fit  :
     abstractElement_ = new GWt_fit(abstractElement, this);
     break;
     case inconnu :
     abstractElement_ = NULL;
     }
     if (abstractElement_ != NULL) {
         dragElementZone_->addWidget(abstractElement_->getImage());
         dragElementZone_->setMaximumSize(32,32);
         dragElementZone_->setMinimumSize(32,32);
     }
     }
     // add second line
     if (abstractElement == NULL) {
     elementName_ = new WText();
     } else {
     elementName_ = new WText(abstractElement->getLabel());
     }
     
     gridLayout->addWidget	(elementName_,1,0,1,3);
    
    decorationStyle().setBorder (WBorder (WBorder::Dotted));
    
    // first insert message
    if (abstractElement == NULL) {
        firstTimeInsertMessage_ = new WText("Drag elements here");
     
         WVBoxLayout* fit = new WVBoxLayout();
         fit->addWidget(firstTimeInsertMessage_,1,Wt::AlignCenter |Wt::AlignMiddle);
         dragElementZone_->setLayout(fit);
    }
    
    //  dragElementZone_->addWidget(firstTimeInsertMessage_);
    //  gridLayout->addWidget	(new WText("Drag elements here"),0,0);
    
    // set layout
    setLayout(gridLayout);
    //addWidget(new WText("Drag elements here"));
}


GWt_elementLigneFaiseau::~GWt_elementLigneFaiseau()
{
}


void GWt_elementLigneFaiseau::onClick(GWt_abstractElement* gw)
{
  cout << "mouse key was double clicked on this widget " << gw->getWidget()->id()<< endl;

  messageDropZone_ = new GWt_dialog("Question","Are you sure you want to delete the selected item?",GWt_dialog::Warning,true,true);

  WSignalMapper<GWt_abstractElement *> *MyMap = new WSignalMapper<GWt_abstractElement *>(this);
  MyMap->mapped().connect(this, &GWt_elementLigneFaiseau::removeDone);
  MyMap->mapConnect(messageDropZone_->finished(),gw);
  messageDropZone_->exec();
}


void GWt_elementLigneFaiseau::removeDone(GWt_abstractElement* gw)
{
  if (messageDropZone_->result() == WDialog::Accepted) {    
    // FIXME
    //    removeDropZone();
  }
}



void GWt_elementLigneFaiseau::removeOtherDropZone(GWt_elementLigneFaiseau* other)
{
  // 3 cases :
  // - first element : remove elem + link after
  // - last element : remove elem + link before
  // - middle element : remove elem + link after/before

  WContainerWidget* wc = static_cast <WContainerWidget*> (parent());
  WBoxLayout* layout = ((WBoxLayout*)(wc->layout()));
  
#ifdef PSPA_DEBUG
  if ( static_cast<GWt_console*> (wApp->findWidget ("console"))) {
    GWt_console* console = static_cast<GWt_console*> (wApp->findWidget ("console"));
    console->addConsoleMessage("GWt_elementLigneFaiseau::mouseOut");
    console->addConsoleMessage("remove from drag Zone");
  }
  cout << "remove from drag Zone id" << layout->indexOf ((WLayoutItem*)(layout->findWidgetItem (other))) << " count:" << layout->count()<< endl;
#endif
  // first element
  if (layout->indexOf ((WLayoutItem*)(layout->findWidgetItem (other))) == 1) {
    if (layout->itemAt (2)) {
      layout->removeItem (layout->itemAt(2));
#ifdef PSPA_DEBUG
      if ( static_cast<GWt_console*> (wApp->findWidget ("console"))) {
        GWt_console* console = static_cast<GWt_console*> (wApp->findWidget ("console"));
	console->addConsoleMessage(" remove first link ");
      }
#endif
      }
    
      // last and middle
    } else if (layout->indexOf ((WLayoutItem*)(layout->findWidgetItem (other))) == layout->count()-2) {
      if (layout->itemAt (layout->count()-3)) {
	layout->removeItem (layout->itemAt(layout->count()-3));
#ifdef PSPA_DEBUG
	if ( static_cast<GWt_console*> (wApp->findWidget ("console"))) {
	  GWt_console* console = static_cast<GWt_console*> (wApp->findWidget ("console"));
	  console->addConsoleMessage(" remove first link ");
    }
#endif
	}
      }
  
      // remove the drop zone
      if (layout->findWidgetItem (other)) {
	layout->removeItem (layout->itemAt(layout->indexOf((WLayoutItem*)(layout->findWidgetItem (other)))));
#ifdef PSPA_DEBUG
	if ( static_cast<GWt_console*> (wApp->findWidget ("console"))) {
	  GWt_console* console = static_cast<GWt_console*> (wApp->findWidget ("console"));
	  console->addConsoleMessage(" remove drop zone ");
    }
#endif
	}

  
	// FIXME ne mas passer par un label mais par un abstrastElement
	//  pspa_->getDataManager()->getCollection()->eraseElement(label);
	pspa_->updateSelections();
}


void GWt_elementLigneFaiseau::addElement(GWt_abstractElement* elem){

  clear();
  abstractElement_ = elem;
  addWidget(abstractElement_);
  if (static_cast <WContainerWidget*> (parent())) {
    WContainerWidget* wc = static_cast <WContainerWidget*> (parent());
    WBoxLayout* layout = ((WBoxLayout*)(wc->layout()));
    layout->setStretchFactor(this,0);		
  }


  firstTimeInsertMessage_ = NULL;
  arrowImage_ = NULL;
}

std::string GWt_elementLigneFaiseau::getElementNumberInBeamLine(nomdElements elem){
  string str = "";
  
  if (static_cast <WContainerWidget*> (parent())) {
    WContainerWidget* wc = static_cast <WContainerWidget*> (parent());
    for (int a=0; a<wc->count(); a++) {
      if (static_cast <GWt_abstractElement*> (wc->widget (a))) {
//        GWt_abstractElement* ab = static_cast <GWt_abstractElement*> (wc->widget (a));
        /*
        printf("1--->%d\n",ab->getElement());
        printf("2--->%d\n",ab->getElement()->getLenghtOfElement());
        printf("3--->%d\n",ab->getElement()->getName().getElementType());
        printf("4--->%d\n",elem.getElementType());

        if (static_cast <GWt_abstractElement*> (wc->widget (a))->getElement()->getName().getElementType() == elem.getElementType()) {
          pspa_->addConsoleMessage("type trouv");
        } else {
          pspa_->addConsoleMessage("type trouv NOT ");
        }
        */

      }
    }
    
    int n = 999999;
    
    std::stringstream stream;
    stream << n ;
    std::string number(stream.str());
    
    if(n < 10) {
      str= elem.getElementLabel()+"0"+number;
    } else {
    str= elem.getElementLabel()+number;
    }
  }

  return str;
}

void GWt_elementLigneFaiseau::setElementName(std::string label) {
    elementName_->setText(label);
}

